<?php

namespace EventPortal\Domain\ActionList\Handlers;

use Closure;
use EventPortal\Domain\ActionList\ActionList;
use EventPortal\Domain\ActionList\Facades\ActionListTemplate;
use EventPortal\Domain\ActionList\Models\ActionList as ActionListModel;
use EventPortal\Domain\ActionList\Task;
use EventPortal\Domain\Form\Payload;

class AssignActionLists
{
    /**
     * @return mixed
     */
    public function __invoke(Payload $payload, Closure $next)
    {
        $event = $payload->getModel();

        $template = ActionListTemplate::make($event->typeValue)->get();

        $template
            ->lists
            ->each(function (ActionList $actionList) use ($event): void {
                $model = ActionListModel::firstOrCreate([
                    'event_id' => $event->id,
                    'status' => $actionList->status,
                    'type' => $actionList->type,
                ]);

                $actionList->tasks->each(function (Task $task) use ($model): void {
                    $model->tasks()->create([
                        'name' => $task->name,
                        'label' => $task->label,
                        'required' => $task->required,
                        'action' => $task->getAction(),
                    ]);
                });
            });

        return $next($payload);
    }
}
