<?php

namespace EventPortal\Domain\Delegate;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\DisplayForm;
use EventPortal\Domain\Form\EditForm;

class DelegateForm
{
    /** @var Delegate */
    private $delegate;

    /** @var Form */
    private $form;

    /** @var Event */
    private $event;

    public static function make(): self
    {
        return new self();
    }

    public function forDelegate(Delegate $delegate): self
    {
        $delegate->loadMissing(['formSteps']);

        $this->delegate = $delegate;

        return $this;
    }

    public function forForm(Form $form): self
    {
        $this->form = $form;

        return $this;
    }

    public function forEvent(Event $event): self
    {
        $this->event = $event;

        return $this;
    }

    public function forDisplaying(): array
    {
        return DisplayForm::make()
            ->forForm($this->form)
            ->forDelegate($this->delegate)
            ->render();
    }

    public function forEditing(?FormStep $step = null): array
    {
        $currentStep = $step ?: $this->currentStep();

        return EditForm::make()
            ->forForm($this->form)
            ->forDelegate($this->delegate)
            ->forEvent($this->event)
            ->currentStep($currentStep)
            ->previousStep($this->previousStep($currentStep))
            ->render();
    }

    public function currentStep(): FormStep
    {
        $partial = $this
            ->delegate
            ->formSteps()
            ->forForm($this->form)
            ->where('state', State::Partial()->getValue())
            ->orderBy('delegate_form_step.updated_at', 'desc')
            ->first();

        if ($partial !== null) {
            return $partial;
        }

        $completed = $this->delegate->formSteps()->forForm($this->form)->first();

        if ($completed !== null) {
            $step = $completed;

            return $this->nextStep($step);
        }

        return $this->form->steps->first();
    }

    public function nextStep(FormStep $step): ?FormStep
    {
        if ($step->isLast() === true) {
            return $step;
        }

        return FormStep::forForm($this->form)
            ->where('order', $step->order + 1)
            ->first();
    }

    public function previousStep(FormStep $step): ?FormStep
    {
        return FormStep::forForm($this->form)
            ->where('order', $step->order - 1)
            ->first();
    }

    public function getDelegateTransition(): string
    {
        if ($this->allStepsCompleted()) {
            return $this->getCompletedTransition();
        }

        if ($this->form->isTypeDecline()) {
            return 'declining';
        }

        return 'registering';
    }

    private function getCompletedTransition(): string
    {
        return $this->form->isTypeDecline() ? 'declined' : 'registered';
    }

    public function allStepsCompleted(): bool
    {
        $formStepsDelegateHasCompleted = $this
            ->delegate
            ->formSteps()
            ->forForm($this->form)
            ->complete()
            ->count();

        return $this->form->steps()->count() === $formStepsDelegateHasCompleted;
    }
}
