<?php

namespace EventPortal\Domain\Event;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\RegistrationStatus;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Arr;

class RegistrationCallToAction
{
    /** @var \EventPortal\Contracts\Event */
    private $event;

    /** @var \EventPortal\Domain\Event\Models\EventDelegate */
    private $eventDelegate;

    /** @var null|Form */
    private $form;

    public static function make(): self
    {
        return new self();
    }

    public function forEventDelegate(EventDelegate $eventDelegate): self
    {
        $this->eventDelegate = $eventDelegate;
        $this->event = $eventDelegate->event;
        $this->form = $this->getForm($eventDelegate);

        return $this;
    }

    /** Get the button */
    public function get(): ?Button
    {
        if ($this->form === null) {
            return null;
        }

        $button = Button::make($this->buttonLabel($this->eventDelegate->status))
            ->icon('chevron-right', 'right')
            ->method(InertiaMethod::VISIT())
            ->route('delegate.form.index', [
                'event' => $this->event->slug,
                'form' => optional($this->form)->slug,
            ]);

        return $button;
    }

    public function getAsArray(): array
    {
        return array_filter([$this->get()]);
    }

    private function buttonLabel(string $status): string
    {
        $labels = [
            'invited' => 'Begin registration',
            'registering' => 'Continue registration',
            'registered' => 'View registration details',
            'declined' => 'View reason for declining',
        ];

        return Arr::get($labels, $status, 'Begin registration');
    }

    private function getForm(EventDelegate $eventDelegate): ?Form
    {
        return Form::forEvent($eventDelegate->event)
            ->when(
                $eventDelegate->statusIs([RegistrationStatus::declining(), RegistrationStatus::declined()]),
                function (Builder $query): Builder {
                    return $query->decline();
                },
                function (Builder $query): Builder {
                    return $query->registration();
                }
            )
            ->first();
    }
}
