<?php

namespace EventPortal\Tests\Feature\Delegate\Event;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class EventLandingTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test **/
    public function a_delegate_can_view_the_landing_page_for_one_of_their_events()
    {
        $event = factory(Event::class)
            ->state('snapshot')
            ->create();

        factory(Form::class)->state('snapshot')->create(['event_id' => $event->id]);
        factory(Form::class)->states(['snapshot', 'decline'])->create(['event_id' => $event->id]);

        $delegate = factory(EventDelegate::class)
            ->state('invited')
            ->create([
                'event_id' => $event->id,
                'delegate_id' => function () {
                    return factory(Delegate::class)
                        ->state('snapshot')
                        ->create()
                        ->id;
                },
            ]);

        $this->assertMatchesJsonSnapshot(
            $this
                ->actingAs($delegate->delegate->user)
                ->get(route('delegate.landing', $event->slug))
                ->props()
        );
    }

    /** @test **/
    public function a_delegate_cannot_view_the_landing_page_for_another_event()
    {
        $this->markTestSkipped('Awaiting middleware');

        $event = factory(Event::class)->create();
        $delegate = factory(EventDelegate::class)->create();

        $this->assertNotEquals($event->id, $delegate->event_id);

        $this
            ->actingAs($delegate->delegate->user)
            ->get(route('delegate.landing', $event->slug))
            ->assertStatus(401);
    }
}
