<?php

namespace EventPortal\Tests\Unit\Event\Form\Models;

use EventPortal\Domain\Event\Form\Enums\FormType;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class FormTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_get_form_types(): void
    {
        $this->assertEquals([
            [
                'label' => 'Registration',
                'value' => 'registration',
            ],
            [
                'label' => 'Decline',
                'value' => 'decline',
            ],
        ], Form::typesAsOptions());
    }

    /** @test **/
    public function can_be_deleted()
    {
        $event = factory(Event::class)->state('draft')->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);

        $this->assertTrue($form->canBeDeleted());
    }

    /** @test **/
    public function cannot_be_deleted()
    {
        $event = factory(Event::class)->state('published')->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);

        $this->assertFalse($form->canBeDeleted());
    }
}
