<?php

namespace EventPortal\Tests\Unit\Rules;

use EventPortal\Domain\Agenda\Actions\CreateDefaultAgenda;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Agenda\Rules\NoOverlap;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Carbon;

class NoOverlapTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function only_validates_for_pathway_and_agenda_given()
    {
        $event = factory(Event::class)->create();

        $pathway = factory(Pathway::class)
            ->state('default')
            ->create([
                'event_id' => $event->id,
            ]);

        (new CreateDefaultAgenda)->execute($event, '2020-05-20');
        (new CreateDefaultAgenda)->execute($event, '2020-05-21');

        $agendaA = Agenda::find(1);
        $agendaB = Agenda::find(2);

        factory(Session::class)->create([
            'starting_at' => Carbon::parse('2020-05-20 13:00:00'),
            'ending_at' => Carbon::parse('2020-05-20 14:00:00'),
            'agenda_id' => $agendaA->id,
            'pathway_id' => $pathway->id,
        ]);

        $rule = (new NoOverlap($pathway, $agendaB));

        $this->assertTrue($rule->passes('ending_at', '2020-05-21 13:30:00'));
    }
}
