<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Component\Input\Text;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Component\Step;
use EventPortal\Domain\Util\ValidationRules;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Validation\Rule;

class ValidationRulesTest extends TestCase
{
    use DatabaseMigrations;

    /** @test **/
    public function get_validation_rules_when_there_are_steps()
    {
        $fields = [
            Step::make()->fields([
                Section::make('Contact Details')->fields([
                    Text::make('Name')->rules(['required']),
                ]),
            ]),
        ];

        $actual = ValidationRules::for($fields)->get();

        $expected = [
            'name' => ['required'],
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function get_validation_rules_when_there_are_sections()
    {
        $fields = [
            Section::make('Contact Details')->fields([
                Text::make('Name')->rules(['required']),
            ]),
        ];

        $actual = ValidationRules::for($fields)->get();

        $expected = [
            'name' => ['required'],
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function get_validation_rules_where_there_is_a_seciton_and_not()
    {
        $fields = [
            Text::make('Name')->rules(['required']),
            Section::make('Contact Details')->fields([
                Text::make('Email')->rules(['required', 'email']),
                Text::make('Phone')->rules(['required', 'min:5', 'max:10']),
            ]),
        ];

        $actual = ValidationRules::for($fields)->get();

        $expected = [
            'name' => ['required'],
            'email' => ['required', 'email'],
            'phone' => ['required', 'min:5', 'max:10'],
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function can_determine_if_reresolving_is_required()
    {
        $fields = [
            Section::make('Contact Details')->fields([
                Text::make('Name')->rules(Rule::unique('users', 'email')->ignore(1)),
            ]),
        ];

        $rules = ValidationRules::for($fields)->requiresReResolving();

        $this->assertTrue($rules);
    }
}
